#!/bin/bash

#
# harbian-audit for Debian GNU/Linux 7/8/9  Hardening
#

#
# 13.20 Ensure shadow group is empty (Scored)
#

set -e # One error, it's over
set -u # One variable unset, it's over

HARDENING_LEVEL=1

ERRORS=0
FILEGROUP='/etc/group'
PATTERN='^shadow:x:[[:digit:]]+:'

# This function will be called if the script status is on enabled / audit mode
audit () {
	if [ $OS_RELEASE -eq 1 ]; then
    	does_pattern_exist_in_file $FILEGROUP $PATTERN
    	if [ $FNRET = 0 ]; then
        	info "shadow group exists"
        	RESULT=$(grep -E "$PATTERN" $FILEGROUP | cut -d: -f4)
        	GROUPID=$(getent group shadow | cut -d: -f3)
        	debug "$RESULT $GROUPID"
        	if [ ! -z "$RESULT" ]; then
            	crit "Some users belong to shadow group: $RESULT"
				FNRET=1
        	else
            	ok "No user belongs to shadow group"
				FNRET=0
        	fi

        	info "Checking if a user has $GROUPID as primary group"
        	RESULT=$(awk -F: '($4 == shadowid) { print $1 }' shadowid=$GROUPID /etc/passwd)
        	if [ ! -z "$RESULT" ]; then
            	crit "Some users have shadow id as their primary group: $RESULT"
				FNRET=2
        	else
            	ok "No user has shadow id as their primary group"
				FNRET=0
        	fi
    	else
        	crit "shadow group doesn't exist"
			FNRET=3
    	fi
	elif [ $OS_RELEASE -eq 2 ]; then
		ok "shadow group doesn't exist in CentOS 8"
		FNRET=0
	else
		:
	fi
}

# This function will be called if the script status is on enabled mode
apply () {
	if [ $FNRET = 0 ]; then
		ok "Pass."
	else 
    	warn "Editing automatically users/groups may seriously harm your system, report only here"
	fi
}

# This function will check config parameters required
check_config() {
    :
}

# Source Root Dir Parameter
if [ -r /etc/default/cis-hardening ]; then
    . /etc/default/cis-hardening
fi
if [ -z "$CIS_ROOT_DIR" ]; then
     echo "There is no /etc/default/cis-hardening file nor cis-hardening directory in current environment."
     echo "Cannot source CIS_ROOT_DIR variable, aborting."
    exit 128
fi

# Main function, will call the proper functions given the configuration (audit, enabled, disabled)
if [ -r $CIS_ROOT_DIR/lib/main.sh ]; then
    . $CIS_ROOT_DIR/lib/main.sh
else
    echo "Cannot find main.sh, have you correctly defined your root directory? Current value is $CIS_ROOT_DIR in /etc/default/cis-hardening"
    exit 128
fi
