#!/bin/bash

#
# harbian-audit for Debian GNU/Linux 7/8/9/10 or CentOS Hardening
#

#
# 9.1.1 Enable cron Daemon (Scored)
# Modify by: Samson-W (sccxboy@gmail.com)
#

set -e # One error, it's over
set -u # One variable unset, it's over

HARDENING_LEVEL=3

PACKAGE="cron"
SERVICE_NAME="cron"

PACKAGE_CENTOS="cronie"
SERVICE_NAME_CENTOS="crond"

# This function will be called if the script status is on enabled / audit mode
audit () {
	if [ $OS_RELEASE -eq 2 ]; then
		PACKAGE=$PACKAGE_CENTOS	
		SERVICE_NAME=$SERVICE_NAME_CENTOS
	fi
    is_pkg_installed $PACKAGE
    if [ $FNRET != 0 ]; then
        crit "$PACKAGE is not installed!"
    else
        ok "$PACKAGE is installed"
        is_service_enabled $SERVICE_NAME
        if [ $FNRET = 0 ]; then
            ok "$SERVICE_NAME is enabled"
        else
            crit "$SERVICE_NAME is disabled"
        fi
    fi
}

# This function will be called if the script status is on enabled mode
apply () {
	if [ $OS_RELEASE -eq 2 ]; then
		PACKAGE=$PACKAGE_CENTOS	
		SERVICE_NAME=$SERVICE_NAME_CENTOS
	fi
    is_pkg_installed $PACKAGE
    if [ $FNRET = 0 ]; then
        ok "$PACKAGE is installed"
    else
        crit "$PACKAGE is absent, installing it"
		if [ $OS_RELEASE -eq 2 ]; then
			yum install -y $PACKAGE
		else
        	apt_install $PACKAGE
		fi
        is_service_enabled $SERVICE_NAME
        if [ $FNRET != 0 ]; then
            info "Enabling $SERVICE_NAME"
			is_debian_9 
			if [ $FNRET = 0 -o $OS_RELEASE -eq 2 ]; then
            	systemctl enable $SERVICE_NAME > /dev/null 2>&1
				systemctl start $SERVICE_NAME > /dev/null 2>&1
			else
				update-rc.d $SERVICE_NAME remove > /dev/null 2>&1
            	update-rc.d $SERVICE_NAME defaults > /dev/null 2>&1
			fi
        else
            ok "$SERVICE_NAME is enabled"
        fi
    fi 
}

# This function will check config parameters required
check_config() {
    :
}

# Source Root Dir Parameter
if [ -r /etc/default/cis-hardening ]; then
    . /etc/default/cis-hardening
fi
if [ -z "$CIS_ROOT_DIR" ]; then
     echo "There is no /etc/default/cis-hardening file nor cis-hardening directory in current environment."
     echo "Cannot source CIS_ROOT_DIR variable, aborting."
    exit 128
fi

# Main function, will call the proper functions given the configuration (audit, enabled, disabled)
if [ -r $CIS_ROOT_DIR/lib/main.sh ]; then
    . $CIS_ROOT_DIR/lib/main.sh
else
    echo "Cannot find main.sh, have you correctly defined your root directory? Current value is $CIS_ROOT_DIR in /etc/default/cis-hardening"
    exit 128
fi
